#ifndef STRINGKERNEL_H
#define STRINGKERNEL_H

#include "DataType.h"
#include "ErrorCode.h"
#include "ESA.h"
#include "I_SAFactory.h"
#include "I_LCPFactory.h"
#include "I_WeightFactory.h"

#include "W_msufsort.h"
#include "W_kasai_lcp.h"

#include "ConstantWeight.h"
#include "ExpDecayWeight.h"
#include "BoundedRangeWeight.h"
#include "KSpectrumWeight.h"


//' Types of substring weighting functions
enum WeightFunction{CONSTANT, EXPDECAY, KSPECTRUM, BOUNDRANGE};

using namespace std;

class StringKernel {


	public:
		/// Variables
		ESA				      *esa;
		I_WeightFactory	*weigher;
		Real            *val;  //' val array. Storing precomputed val(t) values.
		Real			      *lvs;  //' leaves array. Storing weights for leaves.


		/// Constructors
		StringKernel();

		//' Given contructed suffix array
		StringKernel(ESA *esa_, int weightfn, Real param, int verb=INFO);

		//' Given text, build suffix array for it
		StringKernel(const UInt32 &size, SYMBOL *text, int weightfn, Real param, int verb=INFO);


		/// Destructor
		virtual ~StringKernel();

		//' Methods

		/// Precompute the contribution of each intervals (or internal nodes)
		void PrecomputeVal();

		/// Compute Kernel matrix
		void Compute_K(SYMBOL *xprime, const UInt32 &xprime_len, Real &value);

		/// Set leaves array, lvs[]
		void Set_Lvs(const Real *leafWeight, const UInt32 *len, const UInt32 &m);

		/// Set leaves array as lvs[i]=i for i=0 to esa->length
		void Set_Lvs();

	private:

		int _verb;

		/// An iterative auxiliary function used in PrecomputeVal()
		void IterativeCompute(const UInt32 &left, const UInt32 &right);

};
#endif
